""" 
Author:		 Muhammad Tahir Rafique
Date:		 2022-11-29 15:59:16
Project:	 Django-File-Explorer
Description: Provide about view.
"""

from django import http
from django.views.generic.base import TemplateView
from django.urls import reverse

from explorer import __version__

class About(TemplateView):
    template_name = 'explorer/about.html'
    http_method_names = ['get']
    
    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return http.HttpResponseRedirect(f'/admin/login/?next={reverse("explorer-about")}')
        return super().dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        # GETTING CONTEXT
        context = self.get_context_data()
        context['version'] = __version__
        return self.render_to_response(context)


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-01-05 10:50:48
Project:	 File Explorer
Description: Provide function to extract the data from the location.
"""

import os
import datetime

def argsort(seq):
    return sorted(range(len(seq)), key=seq.__getitem__)

def sort_files(sort_array, array_to_sort, mode='a'):
    """Sort the second list accoring to first list.
    mode: a (assending), d (decending)
    """
    # GETTING SORTING IDX
    sort_idx = argsort(sort_array)
    if mode == 'd':
        sort_idx = sort_idx[::-1]

    # LOOPING THROUGH TO MAKE NEW LIST
    new_list = []
    for idx in sort_idx:
        new_list.append(array_to_sort[idx])
    return new_list

class LocationDataExtractor():
    def __init__(self, location_path) -> None:
        """
        Extract the data from the location.
        """
        self._location_path = location_path
        return None

    def getLocationPath(self):
        """Getting location path."""
        return self._location_path

    def isLocationFile(self):
        """Check weather the given location is file."""
        return os.path.isfile(self.getLocationPath())

    def getFiles(self):
        """Getting list of file in directory."""
        # GETTING LIST OF DIR
        location_path = self.getLocationPath()
        list_dir = os.listdir(location_path)

        # SEPRATING FILES AND DIRS
        file_list, deactive_file_list, file_ctime_list = [], [], []
        for dir in list_dir:
            path = os.path.join(location_path, dir)
            if os.path.isfile(path): # If file.
                try:
                    # GETTING TIME
                    ctime = os.path.getctime(path)
                    file_ctime_list.append(ctime)
                    ftime = datetime.datetime.fromtimestamp(ctime)
                    creation_time = ftime.strftime("%d-%b-%Y %I:%M %p")

                    # GETTING SIZE
                    size = os.path.getsize(path)

                    # APPENDING INFORMATION
                    file_list.append({'name': dir, 'type': 'file', 'valid': True, 'creation_time': creation_time, 'size': size})
                except:
                    deactive_file_list.append({'name': dir, 'type': 'file', 'valid': False})
        
        # PERFORMING SORTING
        file_list = sort_files(file_ctime_list, file_list, mode='d')
        file_list += deactive_file_list
        return file_list

    def getDirectories(self):
        """Getting list of all directory."""
        # GETTING LIST OF DIR
        location_path = self.getLocationPath()
        list_dir = os.listdir(location_path)

        # SEPRATING FILES AND DIRS
        file_list, deactive_file_list, file_ctime_list = [], [], []
        for dir in list_dir:
            path = os.path.join(location_path, dir)
            if not os.path.isfile(path): # If not a file.
                try:
                    # GETTING TIME
                    ctime = os.path.getctime(path)
                    file_ctime_list.append(ctime)
                    ftime = datetime.datetime.fromtimestamp(ctime)
                    creation_time = ftime.strftime("%d-%b-%Y %I:%M %p")

                    # GETTING SIZE
                    size = os.path.getsize(path)

                    # APPENDING INFORMATION
                    file_list.append({'name': dir, 'type': 'directory', 'valid': True, 'creation_time': creation_time, 'size': size})
                except:
                    deactive_file_list.append({'name': dir, 'type': 'directory', 'valid': False})
        
        # PERFORMING SORTING
        file_list = sort_files(file_ctime_list, file_list, mode='d')
        file_list += deactive_file_list
        return file_list


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-01-03 10:27:24
Project:	 File Explorer
Description: Provide class to validate the user request.
"""

from explorer.views.extractor.request import RequestDataExtractor
from explorer.views.extractor.user import UserDataExtractor

from explorer.views.utils.path import is_path_exists, is_path_contain_reverse

class Validator(RequestDataExtractor, UserDataExtractor):
    def __init__(self, request) -> None:
        """
        Validate the user request. If request is invalid then error is 
        send to frontend. It will validate both GET and POST request.
        """
        RequestDataExtractor.__init__(self, request)
        UserDataExtractor.__init__(self, request.user)

        self._message = None
        self._info = None
        return None

    def getMessage(self):
        """Return the message if exist otherwise return None."""
        return self._message
    
    def updateMessage(self, message):
        """Update the message."""
        self._message = message
        return None

    def getUserInfo(self):
        """Fatch the user information."""
        if self._info is None:
            self._info = self.getInfo()
        return self._info

    def isVolumeValid(self):
        """Check weather the volume is valid or not."""
        # COLLECTING INFORMATION
        info = self.getUserInfo()
        volume = self.getVolume()
        method = self.getRequestMethod()

        # IMPLIMENTING CONDITIONS
        volume_list = list(info.keys())
        valid = False
        if len(volume_list) == 0:
            valid = False
            message = 'No Volume assigned. Please contact admin.'
        elif method == 'GET':
            if volume is None:
                any_ative = False
                for vol in volume_list:
                    if info[vol]['active']:
                        any_ative = True
                        break
                if any_ative:
                    valid =  True
                else:
                    valid = False
                    message = 'All Volumes are deactive.'
            elif volume in volume_list:
                if info[volume]['active']: # Checking volume is active or not
                    if is_path_exists(info[volume]['path']):
                        valid = True
                    else:
                        valid = False
                        message = f'Volume {volume} path does not exists.'
                else:
                    valid = False
                    message = f'Volume {volume} is not active.'
            else:
                valid = False
                message = f'Invalid Volume: {volume}'
        elif method == 'POST':
            if volume in volume_list:
                valid = True
            else:
                valid = False
                message = f'Invalid Volume: {volume}'
        else:
            self.updateMessage(f'Invalid Request Method: {method}')
            valid = False

        # UPDATING MESSAGE
        if not valid:
            self.updateMessage(message)
        return valid

    def isActionValid(self):
        """Check weather the action is valid or not."""
        # COLLECTING INFORMATION
        info = self.getUserInfo()
        volume = self.getVolume()
        action = self.getAction()
        method = self.getRequestMethod()

        # IMPLIMENTING CONDITIONS
        valid = False
        if method == 'GET':
            valid = True
        elif method == 'POST':
            actions = info[volume]['actions']
            if action is None:
                valid = False
            elif action in actions:
                valid = True
            else:
                valid = False
        else:
            self.updateMessage(f'Invalid Request Method: {method}')
            valid = False

        # UPDATING MESSAGE
        if not valid:
            self.updateMessage(f'Invalid Action: {action}')
        return valid

    def isLocationValid(self):
        """Check weather the action is valid or not."""
        # COLLECTING INFORMATION
        info = self.getUserInfo()
        volume = self.getVolume()
        location = self.getLocation()
        method = self.getRequestMethod()

        # IMPLIMENTING CONDITIONS
        valid = False
        if (method == 'GET') or (method == 'POST'):
            if location is None:
                valid = True
            else:
                valid = is_path_exists(info[volume]['path'], location)
                if valid:
                    valid = is_path_contain_reverse(location)
        else:
            self.updateMessage(f'Invalid Request Method: {method}')
            valid = False

        # UPDATING MESSAGE
        if not valid:
            self.updateMessage(f'Invalid Location: {location}')
        return valid

    def isCheckboxValid(self):
        """Check weather the action is valid or not."""
        # COLLECTING INFORMATION
        checbox_idx = self.getCheckboxIdx()
        method = self.getRequestMethod()
        file_data = self.getFileData()

        # IMPLIMENTING CONDITIONS
        valid = False
        if method == 'GET': # Ignore in GET request
            valid = True
        elif method == 'POST':
            if (len(checbox_idx) == 0) and (len(list(file_data.keys())) == 0):
                valid = False
            else:
                valid = True
        else:
            self.updateMessage(f'Invalid Request Method: {method}')
            valid = False

        # UPDATING MESSAGE
        if not valid:
            self.updateMessage(f'Invalid Checkbox Idx')
        return valid

    def isValid(self):
        """Check all request input and validate it."""
        # MAKING SEQUENCE
        sequence = [
            self.isVolumeValid,
            self.isActionValid,
            self.isLocationValid,
            self.isCheckboxValid,
        ]
        
        # PERFORMING OPERATIONS
        for operation in sequence:
            valid = operation()
            if not valid:
                break
        return valid


# Generated by Django 3.2.13 on 2022-10-30 16:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('explorer', '0002_option'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserRole',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('creation_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('opertations', models.ManyToManyField(blank=True, to='explorer.Option')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('volume', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='explorer.volume')),
            ],
            managers=[
                ('userroles', django.db.models.manager.Manager()),
            ],
        ),
    ]



from django import http
from django.http import FileResponse
from django.shortcuts import render
from django.urls import reverse
from django.views.generic.base import TemplateView
from django.conf import settings

from .operations.explorer import ExplorerOperations
from .operations.location import LocationOperations
from .operations.action import ActionOperations
from .logger.logger import ExplorerLogger


class Explorer(TemplateView):
    template_name = 'explorer/index.html'
    http_method_names = ['get', 'post']
    
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.logger = ExplorerLogger()
        return None

    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated:
            return http.HttpResponseRedirect(f'{reverse("explorer-login")}?next={reverse("explorer-main")}')
        
        # GETTING RESPONSE
        response = super().dispatch(request, *args, **kwargs)
        
        # LOGGINF INFORMATION
        self.logger.log(request, response)
        return response
    
    def get(self, request, *args, **kwargs):
        # GETTING OPERATION OBJECT
        xo = ExplorerOperations(request)

        # PERFORMING VALIDATION
        if not xo.isValid():
            return self._render_warning(request, xo.getMessage())

        # GETTING VOLUME AND PATH INFORMATION
        volume_info, location_path = xo.getVolumeInfo(), xo.getLocationPath()
        if xo.getMessage():
            return self._render_warning(request, xo.getMessage())

        # GETTING LOCATION OBJECT
        lo = LocationOperations(volume_info, location_path)

        # IS LOCATION IS FILE
        if lo.isLocationFile():
            return self._render_file(request, lo.getLocationPath())

        # GETTING LOATION REATED DATA
        data_list = lo.getData() # Getting location file and dir data.
        summary_data = lo.getDataSummary() # Getting summary data.
        navigation_bar_data = lo.getNavigationBarData() # Getting navigation bar data.
        
        # GETTING ICONS INFORMATION
        data_list = self._addIconInformation(data_list)

        # GETTING EXPLORER RELATED DATA
        page_data = xo.getPageData(data_list) # Getting page data.
        pagination_data = xo.getPaginationData(page_data) # Getting pagination data.
        
        # MAKING CONTEXT 
        context = self.get_context_data()
        context.update({
            'volume_data': xo.getVolumeData(),
            'action_data': xo.getActionData(),
            'summary_data': summary_data,
            'navigation_bar_data': navigation_bar_data,
            'page_data': page_data,
            'pagination_data': pagination_data
        })
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        # GETTING OPERATION OBJECT
        xo = ExplorerOperations(request)

        # PERFORMING VALIDATION
        if not xo.isValid():
            return self._render_warning(request, xo.getMessage())

        # GETTING VOLUME AND PATH INFORMATION
        volume_info, location_path = xo.getVolumeInfo(), xo.getLocationPath()
        if xo.getMessage():
            return self._render_warning(request, xo.getMessage())

        # GETTING LOCATION OBJECT
        lo = LocationOperations(volume_info, location_path)

        # GETTING LOCATION RELATED DATA
        data_list = lo.getData() # Getting location file and dir data.
        path_list = xo.getCheckedFilePath(data_list) # Getting page data.

        # PERFORMING ACTION
        ao = ActionOperations(volume_info, location_path)
        file_path = ao.performAction(xo.getAction(), path_list, xo.getFileData())
        if ao.getMessage():
            return self._render_warning(request, xo.getMessage())

        if file_path: # Download case
            return FileResponse(open(file_path, 'rb'), as_attachment=True)

        return self._redirect_to_same(request)

    def render_to_response(self, context, **response_kwargs) -> http.HttpResponse:
        
        return super().render_to_response(context, **response_kwargs)

    def _redirect_to_same(self, request):
        """Redirect to same url."""
        return http.HttpResponseRedirect(request.get_full_path())

    def _render_warning(self, request, message):
        """Rendered warning template"""
        return render(request, 'explorer/warning.html', {'message': message})
    
    def _render_error(self, request, message):
        """Rendered error template"""
        return render(request, 'explorer/error.html', {'message': message})

    def _render_file(self, request, file_path):
        """Return the file render response."""
        return FileResponse(open(file_path, 'rb'), as_attachment=False)
    
    def _addIconInformation(self, data_list):
        """Adding icon information if icon package is used."""
        # IF APP IS NOT USED
        if not ('vscode_icons' in settings.INSTALLED_APPS):
            return data_list
        
        # LOADING MODULE IF EXISTS
        try:
            from vscode_icons.vscode import VSCodeIcons
            vsi = VSCodeIcons()
        except:
            return data_list
        
        # UPDATING ICON INFORMATION
        icon_data = []
        for data in data_list:
            # GETTING TYPE AND NAME
            type = data['type']
            name = data['name']
            
            if type == 'directory':
                icon_path = vsi.findDirectoryIcon(name)
            else:
                icon_path = vsi.findFileIcon(name)
            data['icon'] = icon_path
            icon_data.append(data)
        return data_list


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-01-03 12:19:29
Project:	 File Explorer
Description: Provide path related functions.
"""

import os

def is_path_exists(*args):
    """Check weather the path exists or not."""
    # MAKING PATH
    for idx, arg in enumerate(args):
        if idx == 0:
            path = arg
            continue
        path = os.path.join(path, arg)
    return os.path.exists(path)

def is_path_contain_reverse(path):
    """Checking weather the path contain ../"""
    split_path = path.split(os.path.sep)
    if '..' in split_path:
        return False
    return True


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-01-02 11:35:18
Project:	 File Explorer
Description: Provide function to extract usefull information from the request.
"""

class RequestDataExtractor():
    def __init__(self, request) -> None:
        """This class will receive request object and get usefull information from it."""
        self._request = request
        return None
    
    def getRequestMethod(self):
        """Getting request method."""
        return self._request.method

    def _getFieldValue(self, field_name):
        """Getting single value using field name."""
        # GETTING METHOD
        method = self.getRequestMethod()

        # GETTING DATA
        if method == 'GET':
            data = self._request.GET.get(field_name)
        else:
            data = self._request.POST.get(field_name)
        return data

    def getVolume(self):
        """Get volume information from request."""
        return self._getFieldValue('volume')

    def getAction(self):
        """Get action information from request."""
        return self._getFieldValue('action')

    def getLocation(self):
        """Get location information from request."""
        return self._getFieldValue('location')

    def getPageNumber(self):
        """Get page number information from request."""
        return self._getFieldValue('page')

    def getCheckboxIdx(self):
        """Getting list of checkbox idx."""
        check_idx = self._request.POST.getlist('check_idx')
        check_idx = [int(idx) for idx in check_idx]
        return check_idx
    
    def getFileData(self):
        """Getting file data from the request."""
        return self._request.FILES



""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-02-12 12:22:27
Project:	 Explorer App
Description: Provide class for login.
"""
from django import http
from django.views.generic.base import TemplateView
from django.shortcuts import render
from django.contrib.auth import authenticate, login


class Login(TemplateView):
    template_name = 'explorer/login.html'
    http_method_names = ['get', 'post']

    def get(self, request, *args, **kwargs):
        # GETTING CONTEXT
        context = self.get_context_data()
        return self.render_to_response(context)
    
    def post(self, request, *args, **kwargs):
        username = request.POST.get('username')
        password = request.POST.get('password')
        user = authenticate(request, username=username, password=password)
        if user is not None:
            login(request, user)
            return http.HttpResponseRedirect(redirect_to=request.GET.get('next'))
        return render(request, template_name='explorer/error.html', context={'message': 'Username or Password is incorrect.'})


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-01-04 11:40:18
Project:	 File Explorer Lib
Description: Provide class to perform explorer related operations.
"""

import os

from django.core.paginator import Paginator

from explorer.views.validator.request import Validator
from explorer.views.utils.path import is_path_exists

class ExplorerOperations(Validator):
    def __init__(self, request, max_row_on_page=12, max_page_link=3) -> None:
        """
        Perform different operations.
        """
        Validator.__init__(self, request)
        self._max_row_on_page = max_row_on_page
        self._max_page_link = max_page_link
        return None

    def getMaxRowOnPage(self):
        """Getting maximum number of rows."""
        return self._max_row_on_page

    def getMaxPageLink(self):
        """Getting maximum number pager to display on paginator."""
        return self._max_page_link

    def getVolumeInfo(self):
        """Getting information of current selected volume."""
        # GETTING INFORMATION
        volume = self.getVolume()
        info = self.getInfo()

        # IF VOLUME IS GIVEN
        if volume:
            vol_info = info[volume]
            vol_info['name'] = volume
            return vol_info

        # GETTING VOLUME PATH
        vol_info = None
        vol_name = None
        vol_active = False
        for name, value in info.items():
            if value['active']: # Checking is volume active
                vol_active = True # Just for returning correct error if occure.
                if is_path_exists(value['path']): # Checking if path exists or not.
                    vol_name = name
                    vol_info = value
                    break
        
        # IF PATH IS NONE
        if vol_info is None:
            if vol_active:
                self.updateMessage('Active Volumes path does not exists.')
            else:
                self.updateMessage('All Volumes are deactive.')
            return None

        # APPENFING NAME INFORMATION
        vol_info['name'] = vol_name
        return vol_info

    def getLocationPath(self):
        """Get location information from request."""
        # GETTING LOCATION
        location = self.getLocation()

        # ADDING EXCEPTION
        if location is None:
            location = ''
        return location

    def getVolumeData(self):
        """Getting volume data."""
        # GETTING INFORMATION
        info = self.getInfo()
        volume_info = self.getVolumeInfo()

        volume_data = []
        # LOOPING THROUGH EACH VOLUME
        for name, data in info.items():
            # CHECKING PATH
            error = None
            path = data['path']
            if not is_path_exists(path): # Disabling volume if its path does not exists.
                error = 'path does not exists'
            
            # CHECKING ACTIVENESS
            active = data['active']
            if not active:
                error = 'disable by admin' # Disabling volume if it is inactive by admin.
            
            # CHECKING SELECTION
            selected = False
            if name == volume_info['name']:
                selected = True

            # UPDATING INFORMATION
            volume_data.append({
                'name': name,
                'selected': selected,
                'error': error,
                'url': f'?volume={name}'
            })
        return volume_data

    def getActionData(self):
        """Getting action data."""
        # GETTING INFORMATION
        volume_info = self.getVolumeInfo()
        return volume_info['actions']

    def getPageData(self, data_list):
        """Getting data in django pagination data."""
        # GETTING PAGE NUMBER INHARATED FROM PARENT
        page_number = self.getPageNumber()

        # MAKING DJANGO PAGINATOR
        p = Paginator(data_list, self.getMaxRowOnPage())
        page_data = p.get_page(page_number)
        return page_data

    def getPaginationData(self, page_data):
        """Generating context for the paginator."""
        # CHECKING WEATHER THE PAGINATOR REQUIRED
        if page_data.paginator.num_pages == 1:
            return None

        # GETTING VOLUME AND LOCATIO INFORMATION
        volume_name = self.getVolumeInfo()['name']
        location_path = self.getLocationPath()
        
        # EMPTY PAGINATOR DATA
        max_page_link = self.getMaxPageLink()
        paginator_data = {}

        # GETTING PREVIOUS PAGE LINK
        if page_data.has_previous():
            pg_num = page_data.previous_page_number()
            paginator_data['previous_page'] = {'url': f'?volume={volume_name}&location={location_path}&page={pg_num}'}

        # GETTING NEXT PAGE LINK
        if page_data.has_next():
            pg_num = page_data.next_page_number()
            paginator_data['next_page'] = {'url': f'?volume={volume_name}&location={location_path}&page={pg_num}'}

        # GETTING START PAGE LINK
        current_page_number = page_data.number
        mid_point = max_page_link // 2
        start_page_link = current_page_number - mid_point
        if start_page_link < 1:
            start_page_link = 1

        # GETTING LAST PAGE LINK
        last_page_link = current_page_number + mid_point
        if last_page_link > page_data.paginator.num_pages:
            last_page_link = page_data.paginator.num_pages
        
        # LOOPING TO GET MIDDLE PAGES
        middle_pages = []
        for page in range(start_page_link, last_page_link+1):
            # GETTING SELECTED STATUS
            if page == current_page_number:
                selected = 'active'
            else:
                selected = 'deactive'

            middle_pages.append({
                'number': page,
                'selected': selected,
                'url': f'?volume={volume_name}&location={location_path}&page={page}'
            })
        paginator_data['middle_pages'] = middle_pages
        return paginator_data

    def getCheckedFilePath(self, data_list):
        """Getting file paths which are check on frontend."""
        # GETTING CHECKBOX
        check_idx = self.getCheckboxIdx()

        # GETTING PAGE DATA
        page_data = self.getPageData(data_list)

        # GETTING VOLUME AND LOCATON PATH
        volume_path = self.getVolumeInfo()['path']
        location = self.getLocationPath()

        # MAKING FILE PATHS
        path_list = []
        for idx in check_idx:
            # GETTING SINGLE DATA
            data = page_data[idx]
            
            # MAKING PATH
            path_list.append(
                os.path.join(volume_path, location, data['name'])
            )
        return path_list


# Generated by Django 3.2.13 on 2022-11-17 09:49

from django.db import migrations
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('explorer', '0003_userrole'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Option',
            new_name='Action',
        ),
        migrations.AlterModelManagers(
            name='action',
            managers=[
                ('actions', django.db.models.manager.Manager()),
            ],
        ),
    ]



# Generated by Django 3.2.13 on 2023-02-15 06:22

from django.db import migrations

def add_actions(apps, schema_editor):
    # GETTING EXISTING ACTIONS
    Action = apps.get_model('explorer', 'Action')
    queryset = Action.actions.all()
    action_names_exists = [query.name for query in queryset]
    
    # GETTING LIST OF ADDED ACTION
    added_actions = [
        'download',
        'delete',
        'upload',
        'unzip'
    ]
    
    # ADDING ACTION TO DB
    for action in added_actions:
        if action in action_names_exists:
            continue
        Action.actions.create(name=action)
    
    return None


class Migration(migrations.Migration):

    dependencies = [
        ('explorer', '0005_rename_opertations_userrole_actions'),
    ]

    operations = [
        migrations.RunPython(add_actions),
    ]



from django.db import models
from django.utils import timezone


class ActionManager(models.Manager):
    def __init__(self) -> None:
        super().__init__()
    
    def getIfExists(self, option_name):
        queryset = self.filter(name=option_name)
        if queryset.exists():
            return queryset.first()
        return None
    
    def addIfNotExist(self, action_name):
        queryset = self.filter(name=action_name)
        if queryset.exists():
            return None
        self.create(name=action_name)
        return None

class Action(models.Model):
    name = models.CharField(max_length=120)
    creation_date = models.DateTimeField(default=timezone.now)

    actions = ActionManager()

    def __str__(self) -> str:
        return self.name

    # GETTERS
    def getName(self):
        """Get the name of option."""
        return self.name


from django.db import models
from django.utils import timezone


class VolumeManager(models.Manager):
    def __init__(self) -> None:
        super().__init__()
    
    def getIfExists(self, volume_name):
        queryset = self.filter(name=volume_name)
        if queryset.exists():
            return queryset.first()
        return None

class Volume(models.Model):
    name = models.CharField(max_length=120)
    path = models.CharField(max_length=2048)
    active = models.BooleanField(default=True)
    creation_date = models.DateTimeField(default=timezone.now)

    volumes = VolumeManager()

    def __str__(self) -> str:
        return self.name

    # GETTERS
    def getName(self):
        """Get the name of volume"""
        return self.name

    def getPath(self):
        """Return the volume path."""
        return self.path

    def isActive(self):
        """Return the active status of volume."""
        return self.active


from django.contrib import admin
from django.contrib.admin import AdminSite
from django.contrib.auth.models import User, Group

from .models import Volume, Action, UserRole

# BASIC
class ExplorerAdmin(AdminSite):
    site_title = 'Explorer Admin'
    site_header = 'Explorer Administration'
    index_title = 'Home Page'

explorer_admin = ExplorerAdmin(name='exploreradmin')

# VOLUMES
@admin.register(Volume, site=explorer_admin)
class VolumeAdmin(admin.ModelAdmin):
    list_display = ['name', 'active', 'creation_date']
    list_display_links = ['name']
    list_filter = ['active', 'creation_date']
    ordering = ('-creation_date',)
    search_fields = ['name', 'active']
    fields = [('name', 'active'), 'path', 'creation_date']

# OPTION
@admin.register(Action, site=explorer_admin)
class ActionAdmin(admin.ModelAdmin):
    list_display = ['name', 'creation_date']
    list_display_links = ['name']
    ordering = ('-creation_date',)
    fields = ['name', 'creation_date']

# USER ROLE
@admin.register(UserRole, site=explorer_admin)
class UserRoleAdmin(admin.ModelAdmin):
    list_display = ['user', 'volume', 'creation_date']
    filter_horizontal = ['actions']
    list_filter = ['user', 'volume']
    ordering = ('-user',)
    search_fields = ['user', 'volume']


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-01-04 10:45:31
Project:	 File Explorer
Description: Provide class to perform location related operations.
"""

import os

from explorer.views.extractor.location import LocationDataExtractor

class LocationOperations(LocationDataExtractor):
    def __init__(self, volume_info, location_path) -> None:
        """Perovide location related operations."""
        # INITALIZING CLASS
        LocationDataExtractor.__init__(self, os.path.join(volume_info['path'], location_path))
        self._volume_name = volume_info['name']
        self._volume_path = volume_info['path']
        self._rel_location_path = location_path
        return None

    def getRelLocationPath(self):
        """Getting relative location path."""
        return self._rel_location_path

    def getVolumePath(self):
        """Getting volume path."""
        return self._volume_path
    
    def getVolumeName(self):
        """Getting volume name."""
        return self._volume_name

    def getData(self):
        """Getting list of files."""
        # GETTING FILE LIST
        file_data = self.getFiles()

        # GETTING DIR LIST
        dir_data = self.getDirectories()

        # APPENDING INFORMATION
        data_list = dir_data + file_data

        # ADDING EXTRA INFORMATION IN DATA
        location = self.getRelLocationPath()
        volume_name = self.getVolumeName()
        for data in data_list:
            # GETTING RELATIVE PATH
            name = data['name']
            rel_path = os.path.join(location, name)

            # MAKING URL
            url = f'volume={volume_name}&location={rel_path}'

            # ADDING INFORMATION
            data['url'] = url
        return data_list

    def getDataSummary(self):
        """Getting summary of the data."""
        # GETTING FILE LIST
        file_data = self.getFiles()

        # GETTING DIR LIST
        dir_data = self.getDirectories()

        # GETTING COUNT
        count = {'file': len(file_data), 'directory': len(dir_data)}
        return count

    def getNavigationBarData(self):
        """Getting navigationbar data."""
        # GRTTING INFORMATION
        location = self.getRelLocationPath()
        volume_name = self.getVolumeName()

        # MAKING NAVIGATION BAR
        sep_locations = location.split(os.path.sep)
        nev_location_list = [{'name': volume_name, 'url': f'volume={volume_name}'}]
        rel_loc = ''
        for loc in sep_locations:
            if loc == '':
                continue
            rel_loc = os.path.join(rel_loc, loc)
            nev_location_list.append({'name': loc, 'url': f'volume={volume_name}&location={rel_loc}'})
        return nev_location_list



""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-01-02 12:01:44
Project:	 File Explorer
Description: Provide function to extract usefull information of current user.
"""

from explorer.models.user_role import UserRole

class UserDataExtractor():
    def __init__(self, user) -> None:
        """This class will receive user object and get usefull information from it."""
        self._user = user
        return None

    def getInfo(self):
        """Getting volume and action information for the current user."""
        # GETTING USER ROLES
        queryset = UserRole.userroles.filter(user=self._user)

        # LOOPING THROUGH EACH QUERY
        info = {}
        for query in queryset:
            # GETTING VOLUME OBJECT
            volume = query.getVolume()

            # ADDING VOLUME INFORMATION
            info[volume.name] = {
                'active': volume.active,
                'path': volume.path
            }

            # GETTING ACTION INFORMATION
            actions = []
            for act in query.getActions().all():
                actions.append(act.name)
            info[volume.name]['actions'] = actions
        return info


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-02-12 12:22:27
Project:	 Explorer App
Description: Provide class for login.
"""

from django.views.generic.base import TemplateView
from django.contrib.auth import logout

class Logout(TemplateView):
    template_name = 'explorer/logout.html'
    http_method_names = ['get']

    def get(self, request, *args, **kwargs):
        # GETTING CONTEXT
        context = self.get_context_data()
        
        # LOGOUT USER
        logout(request)
        return self.render_to_response(context)


from django.db import models
from django.utils import timezone

from django.contrib.auth.models import User
from .action import Action
from .volume import Volume

class UserRoleManager(models.Manager):
    def __init__(self) -> None:
        super().__init__()
    
    def getIfExists(self, **kwargs):
        queryset = self.filter(**kwargs)
        if queryset.exists():
            return queryset.first()
        return None

class UserRole(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    volume = models.ForeignKey(Volume, on_delete=models.CASCADE)
    actions = models.ManyToManyField(Action, blank=True)
    creation_date = models.DateTimeField(default=timezone.now)

    userroles = UserRoleManager()

    def __str__(self) -> str:
        return self.user.username

    # GETTERS
    def getUser(self):
        """Get the user."""
        return self.user

    def getVolume(self):
        """Return the volum."""
        return self.volume

    def getActions(self):
        """Return the operations."""
        return self.actions


""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-02-25 11:21:59
Project:	 Explorer
Description: Provide function to write the log file. 
"""

import os
import datetime

from django.conf import settings

from ..extractor.request import RequestDataExtractor

class ExplorerLogger():
    def __init__(self) -> None:
        # GETTING LOG PATH
        try:
            self.log_dir = settings.EXPLORER.get('log_dir')
            self.log_file = settings.EXPLORER.get('log_file')
            self.disable = True
            if self.log_dir:
                self.log_dir = str(self.log_dir)
                self.disable = False
                os.makedirs(self.log_dir, exist_ok=True)
            if self.log_file:
                self.log_file = str(self.log_file)
                self.disable = False
                os.makedirs(os.path.dirname(self.log_file), exist_ok=True)
        except:
            self.disable = True
        return None
    
    def _getInfo(self, request, response):
        """Making information string."""
        info = {}
        # GETTING TIME INFORMATION
        now = datetime.datetime.now()
        info['time'] = now.strftime("[%d/%b/%Y %I:%M:%S:%p]")
        
        # GETTING OTHER INFO
        info['request_method'] = request.method
        info['status_code'] = response.status_code
        info['username'] = request.user.username
        
        # GETTING APPLICATION RELATED INFOMTAION
        extractor = RequestDataExtractor(request)
        info['action'] = extractor.getAction()
        info['volume'] = extractor.getVolume()
        info['page_number'] = extractor.getPageNumber()
        info['location'] = extractor.getLocation()
        info['check_box'] = extractor.getCheckboxIdx()
        return info
    
    def _infoToSingleLine(self, info):
        """Convert the information dict to single line of string."""
        line_info = ""
        line_info += f"{info['time']}"
        line_info += f" {info['request_method']}"
        line_info += f" | {info['status_code']}"
        line_info += f" | {info['username']}"
        line_info += f" | {info['action']}"
        line_info += f" | {info['volume']}"
        line_info += f" | {info['page_number']}"
        line_info += f" | {info['location']}"
        line_info += f" | {info['check_box']} |"
        line_info += "\n"
        return line_info
    
    def _writeLogFile(self, file_path, log_line):
        """Writing single line to log file."""
        with open(file_path, 'a') as f:
            f.writelines([log_line])
        return None
        
    def _writeToDir(self, info):
        """Writing log on the base of username."""
        # GETTING SINGLE LINE INFORMATION
        line_info = self._infoToSingleLine(info)
        
        # GETTING FILE PATH
        username = info.get('username')
        file_path = os.path.join(self.log_dir, f'{username}.log')
        
        # WRITING FILE INFOMTATION
        self._writeLogFile(file_path, line_info)
        return None
    
    def _writeToFile(self, info):
        """Writing log information to a single file."""
        # GETTING SINGLE LINE INFORMATION
        line_info = self._infoToSingleLine(info)
        
        # WRITING FILE INFOMTATION
        self._writeLogFile(self.log_file, line_info)
        return None
        
    def log(self, request, response):
        """Log the information to desire location."""
        # IF LOGGING IS DISABLED
        if self.disable:
            return None
        
        # GETTING INFORMATION
        info = self._getInfo(request, response)
        
        # LOGGING IN DIRECTORY
        if self.log_dir:
            self._writeToDir(info)
        
        # LOGGINF IN FILE
        if self.log_file:
            self._writeToFile(info)
        return None


# Generated by Django 3.2.13 on 2023-01-02 07:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('explorer', '0004_auto_20221117_1449'),
    ]

    operations = [
        migrations.RenameField(
            model_name='userrole',
            old_name='opertations',
            new_name='actions',
        ),
    ]



from setuptools import setup, find_namespace_packages
from pathlib import Path

from explorer import __version__

# README FILE CONTENT
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='django-file-explorer',
    version=__version__,
    description='Django app to explore directory.',

    long_description=long_description,
    long_description_content_type='text/markdown',
    # url='https://hello.com',

    author='Tahir Rafique',
    author_email='tahirrafiqueasad@gmail.com',
    license='MIT',

    packages=find_namespace_packages(),
    package_data={
        "": ["*.html", "*.css", "*.js", "*.svg", "*.json"],
    },
    install_requires=[
        'requests',
        'vscode-icons',
        'django>=3.2.10',
        'psycopg2-binary>=2.9.5'
    ],

    keywords=["explorer", "file explorer", "directory explorer", "django explorer", 'location explorer'],
    classifiers=[
        "Framework :: Django :: 3.2",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Information Technology",
        "Natural Language :: English",
    ]
)


from django.urls import path

from . import views
from .admin import explorer_admin

urlpatterns = [
    path('admin/', explorer_admin.urls),
    path('about', views.About.as_view(), name='explorer-about'),
    path('login/', views.Login.as_view(), name='explorer-login'),
    path('logout/', views.Logout.as_view(), name='explorer-logout'),
    path('', views.Explorer.as_view(), name='explorer-main'),

]



""" 
Author:		 Muhammad Tahir Rafique
Date:		 2023-02-10 10:47:32
Project:	 Explorer App
Description: Provide function to perform operation on volumes.
"""

import os
import shutil
import tempfile
import datetime
from django.core.files.storage import FileSystemStorage

class ActionOperations():
    def __init__(self, volume_info, location_path) -> None:
        """Constructor."""
        self.volume_info = volume_info
        self.location_path = location_path
        self._message = None
        return None
    
    def getMessage(self):
        """Returning private message."""
        return self._message
    
    def updateMessage(self, message):
        """Update the message if any problem occur."""
        self._message = message
        return None
    
    def getVolumeLocation(self):
        """Making complete location using volume path and relative location."""
        return os.path.join(self.volume_info['path'], self.location_path)
    
    def _deleteFiles(self, path_list):
        """Deleting file or dir."""
        for file_path in path_list:
            try:
                if os.path.isfile(file_path):
                    os.remove(file_path)
                else:
                    shutil.rmtree(file_path, ignore_errors=True)
            except:
                pass
        return None

    def _zipFiles(self, file_paths):
        """Zip file or dir."""
        # CHECKING FILE
        if len(file_paths) == 1:
            # CHECKING FILE OR DIR
            if os.path.isfile(file_paths[0]):
                return file_paths[0]

        # MAKING COPY OF FILE IN TEMP DIR
        tmp_dir = tempfile.gettempdir()
        content_dir = os.path.join(tmp_dir, datetime.datetime.now().strftime('%Y-%m-%d_%H-%M-%S'))
        os.makedirs(content_dir)

        # IF SINGLE DIRECTORY
        if len(file_paths) == 1:
            file_name = os.path.basename(file_paths[0])
            zip_dir = os.path.join(content_dir, file_name)
        else:
            zip_dir = content_dir

        # COPYING FILE TO ZIP DIR
        for file_path in file_paths:
            # GETTING FILE NAME
            file_name = os.path.basename(file_path)

            # COPYING FILES
            dst_path = os.path.join(content_dir, file_name)
            if os.path.isfile(file_path):
                shutil.copy(file_path, dst_path)
            else:
                shutil.copytree(file_path, dst_path)

        # ZIPPING LOCATION
        shutil.make_archive(zip_dir, 'zip', zip_dir)
        return zip_dir + '.zip'
    
    def _saveFile(self, upload_data):
        """Save the single file that is uploaded to define path."""
        # GETTING DAVE DIRECTORY
        root_dir = self.getVolumeLocation()
        
        # SAVING EACH FILE
        upload_files = upload_data.getlist('file_data')
        fs = FileSystemStorage(location=root_dir)
        for file in upload_files:
            fs.save(file.name, file)
        return None
    
    def _unZipFiles(self, file_paths):
        """Unzip provided files."""
        # LOOPING THROUGH EACH FILE
        for file_path in file_paths:
            # EXCEPTION FOR NON ZIP FILE
            if not file_path.endswith('zip'):
                continue
            
            # UNZIPPING
            extract_dir = os.path.dirname(file_path)
            archive_format = 'zip'
            shutil.unpack_archive(file_path, extract_dir, archive_format)
        return None

    def performAction(self, action, path_list, file):
        """Performing action on the paths."""
        # PERFORMING OPERATION
        zip_file_path = None
        if action == 'delete':
            self._deleteFiles(path_list)
        elif action == 'download':
            try:
                zip_file_path = self._zipFiles(path_list)
            except:
                self.updateMessage('Unable to download.')
        elif action == 'upload':
            try:
                self._saveFile(file)
            except:
                self.updateMessage('Unable to upload.')
        elif action == 'unzip':
            try:
                self._unZipFiles(path_list)
            except:
                self.updateMessage('Invalid zip file.')
        else:
            pass
        return zip_file_path


# Generated by Django 3.2.13 on 2022-10-30 16:00

from django.db import migrations, models
import django.db.models.manager
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('explorer', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=120)),
                ('creation_date', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            managers=[
                ('options', django.db.models.manager.Manager()),
            ],
        ),
    ]
